/*
  Name: spi_asm.asm
  Copyright: (c) Kai Scheddin
  Author: Kai Scheddin
  Date: 02.04.07 16:45
  Description: serial peripherial interface controll n init asm for ATMEGA32
*/

#include "spi_config.h"


#ifdef NEED_SPI_SHORTYS
.global spi_clock_set
.func spi_clock_set
spi_clock_set:
in r16,SPCR
in r17,SPSR

cpi r24,2
breq 1f
cpi r24,4
breq 2f
cpi r24,8
breq 3f
cpi r24,16
breq 4f
cpi r24,32
breq 5f
cpi r24,64
breq 6f
cpi r24,128
breq 7f
brne 9f

1:
andi r16,0xfc /*0b11111100*/
ori r17,0x01
rjmp 8f
2:
andi r16,0xfc /*0b11111100*/
andi r17,0xfe
rjmp 8f
3:
ori r16,0x01
andi r16,0xfd /*0b11111101*/
ori r17,0x01
rjmp 8f
4:
ori r16,0x01
andi r16,0xfd /*0b11111101*/
andi r17,0xfe
rjmp 8f
5:
andi r16,0xfe
ori r16,0x02 
ori r17,0x01
rjmp 8f
6:
ori r16,0x03 
ori r17,0x01
rjmp 8f
7:
ori r16,0x03
andi r17,0xfe
rjmp 8f
8:
out SPCR,r16
out SPSR,r17
9:
ret
.endfunc

.global spi_set_master
.func spi_set_master
spi_set_master:
in r16,SPCR
ori r16,8
out SPCR,r16
ret
.endfunc

.global spi_set_slave
.func spi_set_slave
spi_set_slave:
in r16,SPCR
andi r16,0xf7
out SPCR,r16
ret
.endfunc

.global spi_set_endian
.func spi_set_endian
spi_set_endian:
cpi r24,0x02
brge 12f

in r16,SPCR
cpi r24,0x01
breq 10f
andi r16,0xef
rjmp 11f



10:
ori r16,0x10
11:
out SPCR,r16	
12:
ret
.endfunc
#endif

#ifdef NEED_SPI_DIRECTS
.global spi_set_spcr
.func spi_set_spcr
spi_set_spcr:
out SPCR,r24
ret
.endfunc
.global spi_set_spsr
.func spi_set_spsr
spi_set_spsr:
out SPSR,r24
ret
.endfunc
.global spi_set_spdr
.func spi_set_spdr
spi_set_spdr:
out SPDR,r24
ret
.endfunc
.global spi_get_spcr
.func spi_get_spcr
spi_get_spcr:
in r24,SPCR
.endfunc
.global spi_get_spsr
.func spi_get_spsr
spi_get_spsr:
in r24,SPSR
ret
.endfunc
.global spi_get_spdr
.func spi_get_spdr
spi_get_spdr:
in r24,SPDR
ret
.endfunc
#endif

#ifdef NEED_SPI_STANDART
.global spi_sendbyte
.func spi_sendbyte
spi_sendbyte:
13:
/*using sbrc instead of sbic causes error's by assembling*/
in r16,SPSR
sbrc r16,SPIF
rjmp 13b
out SPDR,r24
14:
in r16,SPSR
sbrs r16,SPIF
rjmp 14b
ret
.endfunc



.global spi_standart_init
.func spi_standart_init
spi_standart_init:
/* Set MOSI and SCK output, all others input (also DD_SS=PB4)*/
push r16
push r17
clr r17
sbr r17,DD_MOSI
sbr r17,DD_SCK
out DDR_SPI,r17

//SPI init:
	// - master mode
	// - positive clock phase
	// - msb first
	// - maximum SPI speed (fosc/2)
	// - enable spi
	//SPCR = (0<<CPOL)|(1<<MSTR)|(0<<DORD)|(0<<SPR1)|(0<<SPR0)|(1<<SPE);
	//SPSR = (1<<SPI2X);
in r17,SPCR
cbr r17,CPOL
sbr r17,MSTR
cbr r17,DORD
cbr r17,SPR1
cbr r17,SPR0
sbr r17,SPE
out SPCR,r17
in r17,SPSR
sbr r17,SPI2X
out SPSR,r17


/*Enable SPI, Master, set clock rate fck/16*/
//ldi r17,(1<<SPE)|(1<<MSTR)|(1<<SPR0)
//out SPCR,r16
pop r17
pop r16
ret
.endfunc
#endif


#ifdef NEED_SPI_EXTENDED
.global spi_rcvstring
.func spi_rcvstring
spi_rcvstring:
/*
the allocstring is pointed by r25:r24
the length is given by r23:r22
*/

/*save all used register's to stack*/
push r16
push r17
push r26
push r27
push r30
push r31
clr r31
clr r30
mov r27,r25
mov r26,r24
/*check if spcr,spie is enabled*/
in r16,SPCR
sbrc r16,7
rjmp 17f
ldi r17,0x01
cbr r16,7
out SPCR,r16
/*loop, inc counter, wait untill data rcv'd rejump*/
17:
adiw r30,1
171:
sbis SPSR,SPIF
rjmp 171b
in r16,SPDR
st X+,r16
cp r30,r22
cpc r31,r23
breq 18f
rjmp 17b
   
18: /*if spie was enabled, set it to one again*/
sbrs r17,0
rjmp 19f
sbr r16,7
out SPCR,r16        
19:
        
pop r31
pop r30
pop r27
pop r26
pop r17
pop r16
ret
.endfunc
#endif

#ifdef NEED_SPI_EXTENDED
.global spi_sendstring
.func spi_sendstring
spi_sendstring:
push r16
push r17
push r26
push r27
/*mov pointed addy to X*/
mov r27,r25
mov r26,r24
clr r17
/*while(!(SPSR & (1<<SPIF))){*/
20:
sbis SPSR,SPIF  
rjmp 20b
/* if (char [index++]!=0x00) { send char } else {end}*/
ld r16,X+
cp r16,0x00
breq 21f
out SPDR,r16
rjmp 20b
21:

pop r27
pop r26
pop r17
pop r16
ret
.endfunc
#endif



#ifdef NEED_SPI_EXTENDED
.global spi_rcvstring2
.func spi_rcvstring2
spi_rcvstring2:
 /*
the dump byte is in r24
the allocstring is pointed by r23:r22
the length is given by r21:r20
*/

/*save all used register's to stack*/
push r16
push r17
push r26
push r27
push r30
push r31
clr r31
clr r30
mov r27,r23
mov r26,r22
/*check if spcr,spie is enabled*/
in r16,SPCR
sbrc r16,7
rjmp 22f
ldi r17,0x01
cbr r16,7
out SPCR,r16
/*loop, inc counter, wait untill data rcv'd rejump*/
22:
/* send dump byte*/
sbis SPSR,SPIF
rjmp 22b
out SPDR,r24

adiw r30,1
221:
sbis SPSR,SPIF
rjmp 221b
in r16,SPDR
st X+,r16
cp r30,r20
cpc r31,r21
breq 23f
rjmp 22b
   
23: /*if spie was enabled, set it to one again*/
sbrs r17,0
rjmp 24f
sbr r16,7
out SPCR,r16        
24:
        
pop r31
pop r30
pop r27
pop r26
pop r17
pop r16                      

ret
.endfunc

#endif

#ifdef NEED_SPI_EXTENDED
.global spi_rcvstring3
.func spi_rcvstring3
spi_rcvstring3:
               
 /*
 the master_command byte is stored in r24
the dump byte is in r22
the allocstring is pointed by r21:r20
the length is given by r19:r18
*/

/*save all used register's to stack*/
push r16
push r17
push r26
push r27
push r30
push r31
clr r31
clr r30
mov r27,r21
mov r26,r20
/*check if spcr,spie is enabled*/
in r16,SPCR
sbrc r16,7
rjmp 25f
ldi r17,0x01
cbr r16,7
out SPCR,r16
25:
sbis SPSR,SPIF
rjmp 25b
out SPDR,r24

/*loop, inc counter, wait untill data rcv'd rejump*/
26:
/* send dump byte*/
sbis SPSR,SPIF
rjmp 26b
out SPDR,r22

adiw r30,1
261:
sbis SPSR,SPIF
rjmp 261b
in r16,SPDR
st X+,r16
cp r30,r18
cpc r31,r19
breq 27f
rjmp 26b
   
27: /*if spie was enabled, set it to one again*/
sbrs r17,0
rjmp 28f
sbr r16,7
out SPCR,r16        
28:
        
pop r31
pop r30
pop r27
pop r26
pop r17
pop r16                      

               
ret
.endfunc

#endif


.end
